@@ -5,7 +5,7 @@
 //!
 //! # Cargo features
 //!
-//! Default features: log, upload, rustls, human-panic
+//! Default features: log, upload, rustls
 //!
 //! - log: Configures pretty-env-logger, even though maturin doesn't use logging itself.
 //!
@@ -16,8 +16,6 @@
 //!
 //! - native-tls: Makes ureq use the platform native tls stack
 //!
-//! - human-panic: Adds https://github.com/rust-clique/human-panic
-//!
 //! - password-storage (off by default): Uses the keyring package to store the password. keyring
 //! pulls in a lot of shared libraries and outdated dependencies, so this is off by default, except
 //! for the build on the github releases page.
@@ -53,7 +51,7 @@ mod metadata;
 mod module_writer;
 mod new_project;
 mod project_layout;
-mod pyproject_toml;
+pub mod pyproject_toml;
 mod python_interpreter;
 mod source_distribution;
 mod target;
