@@ -295,15 +295,15 @@ impl WheelWriter {
         project_layout: &ProjectLayout,
         metadata21: &Metadata21,
     ) -> Result<()> {
-        if let Some(python_module) = &project_layout.python_module {
-            let absolute_path = python_module.normalize()?.into_path_buf();
-            if let Some(python_path) = absolute_path.parent().and_then(|p| p.to_str()) {
+        if project_layout.python_module.is_some() || !project_layout.python_packages.is_empty() {
+            let absolute_path = project_layout.python_dir.normalize()?.into_path_buf();
+            if let Some(python_path) = absolute_path.to_str() {
                 let name = metadata21.get_distribution_escaped();
                 let target = format!("{}.pth", name);
                 debug!("Adding {} from {}", target, python_path);
                 self.add_bytes(target, python_path.as_bytes())?;
             } else {
-                println!("⚠️ source code path contains non-Unicode sequences, editable installs may not work.");
+                eprintln!("⚠️ source code path contains non-Unicode sequences, editable installs may not work.");
             }
         }
         Ok(())
@@ -720,6 +720,10 @@ pub fn write_bindings_module(
         }
     };
 
+    if !editable {
+        write_python_part(writer, project_layout, pyproject_toml)
+            .context("Failed to add the python module to the package")?;
+    }
     if let Some(python_module) = &project_layout.python_module {
         if editable {
             let target = project_layout.rust_module.join(&so_filename);
@@ -735,9 +739,6 @@ pub fn write_bindings_module(
                 target.display()
             ))?;
         } else {
-            write_python_part(writer, python_module, pyproject_toml)
-                .context("Failed to add the python module to the package")?;
-
             let relative = project_layout
                 .rust_module
                 .strip_prefix(python_module.parent().unwrap())
@@ -789,14 +790,13 @@ pub fn write_cffi_module(
 ) -> Result<()> {
     let cffi_declarations = generate_cffi_declarations(crate_dir, target_dir, python)?;
 
+    if !editable {
+        write_python_part(writer, project_layout, pyproject_toml)
+            .context("Failed to add the python module to the package")?;
+    }
+
     let module;
-
     if let Some(python_module) = &project_layout.python_module {
-        if !editable {
-            write_python_part(writer, python_module, pyproject_toml)
-                .context("Failed to add the python module to the package")?;
-        }
-
         if editable {
             let base_path = python_module.join(module_name);
             fs::create_dir_all(&base_path)?;
@@ -957,14 +957,13 @@ pub fn write_uniffi_module(
     } = generate_uniffi_bindings(crate_dir, target_dir, target_os)?;
     let py_init = format!("from .{} import *  # NOQA\n", binding_name);
 
+    if !editable {
+        write_python_part(writer, project_layout, pyproject_toml)
+            .context("Failed to add the python module to the package")?;
+    }
+
     let module;
-
     if let Some(python_module) = &project_layout.python_module {
-        if !editable {
-            write_python_part(writer, python_module, pyproject_toml)
-                .context("Failed to add the python module to the package")?;
-        }
-
         if editable {
             let base_path = python_module.join(module_name);
             fs::create_dir_all(&base_path)?;
@@ -1096,34 +1095,47 @@ if __name__ == '__main__':
 /// Adds the python part of a mixed project to the writer,
 pub fn write_python_part(
     writer: &mut impl ModuleWriter,
-    python_module: impl AsRef<Path>,
+    project_layout: &ProjectLayout,
     pyproject_toml: Option<&PyProjectToml>,
 ) -> Result<()> {
-    let python_module = python_module.as_ref();
-    for absolute in WalkBuilder::new(python_module).hidden(false).build() {
-        let absolute = absolute?.into_path();
-        let relative = absolute
-            .strip_prefix(python_module.parent().unwrap())
-            .unwrap();
-        if absolute.is_dir() {
-            writer.add_directory(relative)?;
-        } else {
-            // Ignore native libraries from develop, if any
-            if let Some(extension) = relative.extension() {
-                if extension.to_string_lossy() == "so" {
-                    debug!("Ignoring native library {}", relative.display());
-                    continue;
+    let python_dir = &project_layout.python_dir;
+    let mut python_packages = Vec::new();
+    if let Some(python_module) = project_layout.python_module.as_ref() {
+        python_packages.push(python_module.to_path_buf());
+    }
+    for package in &project_layout.python_packages {
+        let package_path = python_dir.join(package);
+        if python_packages.iter().any(|p| *p == package_path) {
+            continue;
+        }
+        python_packages.push(package_path);
+    }
+
+    for package in python_packages {
+        for absolute in WalkBuilder::new(package).hidden(false).build() {
+            let absolute = absolute?.into_path();
+            let relative = absolute.strip_prefix(python_dir).unwrap();
+            if absolute.is_dir() {
+                writer.add_directory(relative)?;
+            } else {
+                // Ignore native libraries from develop, if any
+                if let Some(extension) = relative.extension() {
+                    if extension.to_string_lossy() == "so" {
+                        debug!("Ignoring native library {}", relative.display());
+                        continue;
+                    }
                 }
+                writer
+                    .add_file(relative, &absolute)
+                    .context(format!("File to add file from {}", absolute.display()))?;
             }
-            writer
-                .add_file(relative, &absolute)
-                .context(format!("File to add file from {}", absolute.display()))?;
         }
     }
 
     // Include additional files
     if let Some(pyproject) = pyproject_toml {
-        let pyproject_dir = python_module.parent().unwrap();
+        // FIXME: in src-layout pyproject.toml isn't located directly in python dir
+        let pyproject_dir = python_dir;
         if let Some(glob_patterns) = pyproject.include() {
             for pattern in glob_patterns
                 .iter()
