@@ -1,14 +1,19 @@
 use crate::module_writer::{add_data, ModuleWriter};
-use crate::{BuildContext, PyProjectToml, SDistWriter};
+use crate::{pyproject_toml::Format, BuildContext, PyProjectToml, SDistWriter};
 use anyhow::{bail, Context, Result};
 use cargo_metadata::{Metadata, MetadataCommand};
 use fs_err as fs;
+use ignore::overrides::Override;
+use normpath::PathExt as _;
 use std::collections::HashMap;
 use std::path::{Path, PathBuf};
 use std::process::Command;
 use std::str;
+use tracing::debug;
 
 const LOCAL_DEPENDENCIES_FOLDER: &str = "local_dependencies";
+const RUST_SRC_FOLDER: &str = "rust_src";
+
 /// Inheritable workspace fields, see
 /// https://github.com/rust-lang/cargo/blob/13ae438cf079da58272edc71f4d4968043dbd27b/src/cargo/util/toml/mod.rs#L1140-L1158
 const WORKSPACE_INHERITABLE_FIELDS: &[&str] = &[
@@ -46,7 +51,7 @@ fn rewrite_cargo_toml(
     root_crate: bool,
 ) -> Result<String> {
     let manifest_path = manifest_path.as_ref();
-    let text = fs::read_to_string(&manifest_path).context(format!(
+    let text = fs::read_to_string(manifest_path).context(format!(
         "Can't read Cargo.toml at {}",
         manifest_path.display(),
     ))?;
@@ -61,10 +66,10 @@ fn rewrite_cargo_toml(
     //                          ^^^^^^^^^^^^^^^^^^ table[&dep_name]["path"]
     // ^^^^^^^^^^^^^ dep_name
     for dep_category in &["dependencies", "dev-dependencies", "build-dependencies"] {
-        if let Some(table) = data.get_mut(*dep_category).and_then(|x| x.as_table_mut()) {
+        if let Some(table) = data.get_mut(dep_category).and_then(|x| x.as_table_mut()) {
             let workspace_deps = workspace_manifest
                 .get("workspace")
-                .and_then(|x| x.get(dep_category))
+                .and_then(|x| x.get("dependencies"))
                 .and_then(|x| x.as_table_like());
             let dep_names: Vec<_> = table.iter().map(|(key, _)| key.to_string()).collect();
             for dep_name in dep_names {
@@ -104,11 +109,13 @@ fn rewrite_cargo_toml(
                             let mut workspace_dep = workspace_dep.clone();
                             // Merge optional and features from the current Cargo.toml
                             if table[&dep_name].get("optional").is_some() {
+                                ensure_dep_is_inline_table(&mut workspace_dep);
                                 workspace_dep["optional"] = table[&dep_name]["optional"].clone();
                             }
                             if let Some(features) =
                                 table[&dep_name].get("features").and_then(|x| x.as_array())
                             {
+                                ensure_dep_is_inline_table(&mut workspace_dep);
                                 let existing_features = workspace_dep
                                     .as_table_like_mut()
                                     .unwrap()
@@ -185,19 +192,32 @@ fn rewrite_cargo_toml(
         // Update workspace members
         if let Some(workspace) = data.get_mut("workspace").and_then(|x| x.as_table_mut()) {
             if let Some(members) = workspace.get_mut("members").and_then(|x| x.as_array_mut()) {
-                let mut new_members = toml_edit::Array::new();
-                for member in members.iter() {
-                    if let toml_edit::Value::String(ref s) = member {
-                        let path = Path::new(s.value());
-                        if let Some(name) = path.file_name().and_then(|x| x.to_str()) {
-                            if known_path_deps.contains_key(name) {
-                                new_members.push(format!("{}/{}", LOCAL_DEPENDENCIES_FOLDER, name));
+                if known_path_deps.is_empty() {
+                    // Remove workspace members when there isn't any path dep
+                    workspace.remove("members");
+                    if workspace.is_empty() {
+                        // Remove workspace all together if it's empty
+                        data.remove("workspace");
+                    }
+                    rewritten = true;
+                } else {
+                    let mut new_members = toml_edit::Array::new();
+                    for member in members.iter() {
+                        if let toml_edit::Value::String(ref s) = member {
+                            let path = Path::new(s.value());
+                            if let Some(name) = path.file_name().and_then(|x| x.to_str()) {
+                                if known_path_deps.contains_key(name) {
+                                    new_members
+                                        .push(format!("{}/{}", LOCAL_DEPENDENCIES_FOLDER, name));
+                                }
                             }
                         }
                     }
-                }
-                if !new_members.is_empty() {
-                    workspace["members"] = toml_edit::value(new_members);
+                    if !new_members.is_empty() {
+                        workspace["members"] = toml_edit::value(new_members);
+                    } else {
+                        workspace.remove("members");
+                    }
                     rewritten = true;
                 }
             }
@@ -224,6 +244,52 @@ fn rewrite_cargo_toml(
     }
 }
 
+/// Make sure that the dep entry is an inline table
+/// e.g. in the form of `{ version = "..." }`
+/// so that we can add entries for `optional` and `features`
+fn ensure_dep_is_inline_table(dep: &mut toml_edit::Item) {
+    if let Some(v) = dep.as_value_mut() {
+        if v.is_str() {
+            let val = std::mem::replace(v, toml_edit::Value::from(false));
+            let mut tab = toml_edit::InlineTable::new();
+            tab.insert("version", val);
+            *v = toml_edit::Value::InlineTable(tab);
+        }
+    }
+}
+
+/// When `Cargo.toml` is outside of the directory containing `pyproject.toml`,
+/// we put Rust crate source to `RUST_SRC_FOLDER` and
+/// update `tool.maturin.manifest-path` in `pyproject.toml`.
+fn rewrite_pyproject_toml(pyproject_toml_path: &Path) -> Result<String> {
+    let text = fs::read_to_string(pyproject_toml_path).context(format!(
+        "Can't read pyproject.toml at {}",
+        pyproject_toml_path.display(),
+    ))?;
+    let mut data = text.parse::<toml_edit::Document>().context(format!(
+        "Failed to parse pyproject.toml at {}",
+        pyproject_toml_path.display()
+    ))?;
+    if let Some(tool) = data.get_mut("tool").and_then(|x| x.as_table_mut()) {
+        if let Some(maturin) = tool.get_mut("maturin").and_then(|x| x.as_table_mut()) {
+            if let Some(manifest_path) = maturin.get_mut("manifest-path") {
+                // original: ../$crate/Cargo.toml or ../../$crate/Cargo.toml
+                // rewrite to: $RUST_SRC_FOLDER/$crate/Cargo.toml
+                let path =
+                    Path::new(manifest_path.as_str().context(
+                        "tool.maturin.manifest-path in pyproject.toml must be a string",
+                    )?);
+                let crate_name = path.parent().unwrap().file_name().unwrap();
+                let new_path = Path::new(RUST_SRC_FOLDER)
+                    .join(crate_name)
+                    .join("Cargo.toml");
+                *manifest_path = toml_edit::value(new_path.to_str().unwrap());
+            }
+        }
+    }
+    Ok(data.to_string())
+}
+
 /// Copies the files of a crate to a source distribution, recursively adding path dependencies
 /// and rewriting path entries in Cargo.toml
 ///
@@ -240,7 +306,7 @@ fn add_crate_to_source_distribution(
     let manifest_path = manifest_path.as_ref();
     let pyproject_toml_path = pyproject_toml_path.as_ref();
     let output = Command::new("cargo")
-        .args(&["package", "--list", "--allow-dirty", "--manifest-path"])
+        .args(["package", "--list", "--allow-dirty", "--manifest-path"])
         .arg(manifest_path)
         .output()
         .with_context(|| {
@@ -265,12 +331,12 @@ fn add_crate_to_source_distribution(
         .map(Path::new)
         .collect();
 
-    let abs_manifest_path = fs::canonicalize(manifest_path)?;
+    let abs_manifest_path = manifest_path.normalize()?.into_path_buf();
     let abs_manifest_dir = abs_manifest_path.parent().unwrap();
     let pyproject_dir = pyproject_toml_path.parent().unwrap();
     let cargo_toml_in_subdir = root_crate
         && abs_manifest_dir != pyproject_dir
-        && abs_manifest_dir.starts_with(&pyproject_dir);
+        && abs_manifest_dir.starts_with(pyproject_dir);
 
     // manifest_dir should be a relative path
     let manifest_dir = manifest_path.parent().unwrap();
@@ -279,69 +345,117 @@ fn add_crate_to_source_distribution(
         .map(|relative_to_manifests| {
             let relative_to_cwd = manifest_dir.join(relative_to_manifests);
             if root_crate && cargo_toml_in_subdir {
-                (relative_to_cwd.clone(), relative_to_cwd)
+                let relative_to_project_root = abs_manifest_dir
+                    .strip_prefix(pyproject_dir)
+                    .unwrap()
+                    .join(relative_to_manifests);
+                (relative_to_project_root, relative_to_cwd)
             } else {
                 (relative_to_manifests.to_path_buf(), relative_to_cwd)
             }
         })
         // We rewrite Cargo.toml and add it separately
         .filter(|(target, source)| {
+            #[allow(clippy::if_same_then_else)]
             // Skip generated files. See https://github.com/rust-lang/cargo/issues/7938#issuecomment-593280660
             // and https://github.com/PyO3/maturin/issues/449
             if target == Path::new("Cargo.toml.orig") || target == Path::new("Cargo.toml") {
                 false
+            } else if matches!(target.extension(), Some(ext) if ext == "pyc" || ext == "pyd" || ext == "so") {
+                // Technically, `cargo package --list` should handle this,
+                // but somehow it doesn't on Alpine Linux running in GitHub Actions,
+                // so we do it manually here.
+                // See https://github.com/PyO3/maturin/pull/1255#issuecomment-1308838786
+                debug!("Ignoring {}", target.display());
+                false
             } else {
                 source.exists()
             }
         })
         .collect();
 
+    let prefix = prefix.as_ref();
+    writer.add_directory(prefix)?;
+
+    let mut cargo_toml_in_rust_src = false;
+
     if root_crate
         && !target_source
             .iter()
             .any(|(target, _)| target == Path::new("pyproject.toml"))
     {
         // Add pyproject.toml to the source distribution
-        // if Cargo.toml is in subdirectory of pyproject.toml directory
         if cargo_toml_in_subdir {
+            // if Cargo.toml is in subdirectory of pyproject.toml directory
             target_source.push((
                 PathBuf::from("pyproject.toml"),
                 pyproject_toml_path.to_path_buf(),
             ));
         } else {
-            bail!(
-                "pyproject.toml was not included by `cargo package`. \
-                     Please make sure pyproject.toml is not excluded or build without `--sdist`"
-            )
+            // if pyproject.toml was not included by `cargo package --list`
+            // (e.g. because it is in a parent directory)
+            // we need to add `cargo package --list` files to a subdirectory
+            // and rewrite `tool.maturin.manifest-path` in pyproject.toml
+            let crate_name = abs_manifest_dir.file_name().unwrap();
+            // check that there isn't already a $RUST_SRC_FOLDER/$crate_name/ folder in python root
+            let rust_src = Path::new(RUST_SRC_FOLDER).join(crate_name);
+            if target_source
+                .iter()
+                .any(|(target, _)| target.starts_with(&rust_src))
+            {
+                bail!(
+                    "Cannot add crate {} to source distribution because there is already a {} folder, consider rename it to avoid conflicts",
+                    crate_name.to_string_lossy(),
+                    rust_src.display(),
+                );
+            }
+            target_source.iter_mut().for_each(|(target, _)| {
+                *target = rust_src.join(&target);
+            });
+            // rewrite `tool.maturin.manifest-path` in pyproject.toml
+            let rewritten_pyproject_toml = rewrite_pyproject_toml(pyproject_toml_path)?;
+            writer.add_bytes(
+                prefix.join("pyproject.toml"),
+                rewritten_pyproject_toml.as_bytes(),
+            )?;
+            cargo_toml_in_rust_src = true;
         }
     }
 
+    let cargo_toml_path = if cargo_toml_in_subdir {
+        let relative_manifest_path = abs_manifest_path.strip_prefix(pyproject_dir).unwrap();
+        prefix.join(relative_manifest_path)
+    } else if cargo_toml_in_rust_src {
+        let crate_name = abs_manifest_dir.file_name().unwrap();
+        prefix
+            .join(RUST_SRC_FOLDER)
+            .join(crate_name)
+            .join(manifest_path.file_name().unwrap())
+    } else {
+        prefix.join(manifest_path.file_name().unwrap())
+    };
+
     let local_deps_folder = if cargo_toml_in_subdir {
         let level = abs_manifest_dir
-            .strip_prefix(pyproject_dir)?
+            .strip_prefix(pyproject_dir)
+            .unwrap()
             .components()
             .count();
         format!("{}{}", "../".repeat(level), LOCAL_DEPENDENCIES_FOLDER)
+    } else if cargo_toml_in_rust_src {
+        format!("../../{}", LOCAL_DEPENDENCIES_FOLDER)
     } else {
         LOCAL_DEPENDENCIES_FOLDER.to_string()
     };
     let rewritten_cargo_toml = rewrite_cargo_toml(
-        &manifest_path,
+        manifest_path,
         workspace_manifest,
         known_path_deps,
         local_deps_folder,
         root_crate,
     )?;
 
-    let prefix = prefix.as_ref();
-    writer.add_directory(prefix)?;
-
-    let cargo_toml = if cargo_toml_in_subdir {
-        prefix.join(manifest_path)
-    } else {
-        prefix.join(manifest_path.file_name().unwrap())
-    };
-    writer.add_bytes(cargo_toml, rewritten_cargo_toml.as_bytes())?;
+    writer.add_bytes(cargo_toml_path, rewritten_cargo_toml.as_bytes())?;
 
     for (target, source) in target_source {
         writer.add_file(prefix.join(target), source)?;
@@ -391,20 +505,24 @@ fn find_path_deps(cargo_metadata: &Metadata) -> Result<HashMap<String, PathBuf>>
 pub fn source_distribution(
     build_context: &BuildContext,
     pyproject: &PyProjectToml,
+    excludes: Option<Override>,
 ) -> Result<PathBuf> {
     let metadata21 = &build_context.metadata21;
     let manifest_path = &build_context.manifest_path;
-    let pyproject_toml_path = fs::canonicalize(&build_context.pyproject_toml_path)?;
+    let pyproject_toml_path = build_context
+        .pyproject_toml_path
+        .normalize()?
+        .into_path_buf();
     let workspace_manifest_path = build_context
         .cargo_metadata
         .workspace_root
         .join("Cargo.toml");
     let workspace_manifest: toml_edit::Document =
-        fs::read_to_string(&workspace_manifest_path)?.parse()?;
+        fs::read_to_string(workspace_manifest_path)?.parse()?;
 
     let known_path_deps = find_path_deps(&build_context.cargo_metadata)?;
 
-    let mut writer = SDistWriter::new(&build_context.out, metadata21)?;
+    let mut writer = SDistWriter::new(&build_context.out, metadata21, excludes)?;
     let root_dir = PathBuf::from(format!(
         "{}-{}",
         &metadata21.get_distribution_escaped(),
@@ -418,6 +536,7 @@ pub fn source_distribution(
         // thus we need to find out its workspace root from `cargo metadata`
         let path_dep_metadata = MetadataCommand::new()
             .manifest_path(path_dep)
+            .verbose(true)
             // We don't need to resolve the dependency graph
             .no_deps()
             .exec()
@@ -434,7 +553,7 @@ pub fn source_distribution(
             } else {
                 if !path_dep_workspace_manifests.contains_key(&path_dep_metadata.workspace_root) {
                     let manifest: toml_edit::Document =
-                        fs::read_to_string(&path_dep_metadata.workspace_root.join("Cargo.toml"))?
+                        fs::read_to_string(path_dep_metadata.workspace_root.join("Cargo.toml"))?
                             .parse()?;
                     path_dep_workspace_manifests
                         .insert(path_dep_metadata.workspace_root.clone(), manifest);
@@ -444,7 +563,7 @@ pub fn source_distribution(
         add_crate_to_source_distribution(
             &mut writer,
             &pyproject_toml_path,
-            &path_dep,
+            path_dep,
             path_dep_workspace_manifest,
             &root_dir.join(LOCAL_DEPENDENCIES_FOLDER).join(name),
             &known_path_deps,
@@ -461,28 +580,83 @@ pub fn source_distribution(
     add_crate_to_source_distribution(
         &mut writer,
         &pyproject_toml_path,
-        &manifest_path,
+        manifest_path,
         &workspace_manifest,
         &root_dir,
         &known_path_deps,
         true,
     )?;
 
-    let manifest_dir = manifest_path.parent().unwrap();
-    let cargo_lock_path = manifest_dir.join("Cargo.lock");
+    let abs_manifest_path = manifest_path.normalize()?.into_path_buf();
+    let abs_manifest_dir = abs_manifest_path.parent().unwrap();
+    let cargo_lock_path = abs_manifest_dir.join("Cargo.lock");
+    let cargo_lock_exists = cargo_lock_path.exists();
+    let workspace_cargo_lock = build_context
+        .cargo_metadata
+        .workspace_root
+        .join("Cargo.lock");
+    let workspace_cargo_lock_exists = workspace_cargo_lock.exists();
     let cargo_lock_required =
         build_context.cargo_options.locked || build_context.cargo_options.frozen;
-    if cargo_lock_required || cargo_lock_path.exists() {
-        writer.add_file(root_dir.join("Cargo.lock"), &cargo_lock_path)?;
+    if cargo_lock_required || cargo_lock_exists || workspace_cargo_lock_exists {
+        let project_root = pyproject_toml_path.parent().unwrap();
+        let relative_cargo_lock = if cargo_lock_path.starts_with(project_root) {
+            cargo_lock_path.strip_prefix(project_root).unwrap()
+        } else {
+            cargo_lock_path.strip_prefix(abs_manifest_dir).unwrap()
+        };
+        if cargo_lock_exists {
+            writer.add_file(root_dir.join(relative_cargo_lock), &cargo_lock_path)?;
+        } else {
+            // Fallback to workspace Cargo lock file
+            writer.add_file(root_dir.join(relative_cargo_lock), workspace_cargo_lock)?;
+        }
     } else {
-        println!(
+        eprintln!(
             "⚠️  Warning: Cargo.lock is not found, it is recommended \
             to include it in the source distribution"
         );
     }
 
-    // Add readme, license and python source files
     let pyproject_dir = pyproject_toml_path.parent().unwrap();
+    // Add python source files
+    let mut python_packages = Vec::new();
+    if let Some(python_module) = build_context.project_layout.python_module.as_ref() {
+        python_packages.push(python_module.to_path_buf());
+    }
+    for package in &build_context.project_layout.python_packages {
+        let package_path = build_context.project_layout.python_dir.join(package);
+        if python_packages.iter().any(|p| *p == package_path) {
+            continue;
+        }
+        python_packages.push(package_path);
+    }
+
+    for package in python_packages {
+        for entry in ignore::Walk::new(package) {
+            let source = entry?.into_path();
+            // Technically, `ignore` crate should handle this,
+            // but somehow it doesn't on Alpine Linux running in GitHub Actions,
+            // so we do it manually here.
+            // See https://github.com/PyO3/maturin/pull/1187#issuecomment-1273987013
+            if source
+                .extension()
+                .map(|ext| ext == "pyc" || ext == "pyd" || ext == "so")
+                .unwrap_or_default()
+            {
+                debug!("Ignoring {}", source.display());
+                continue;
+            }
+            let target = root_dir.join(source.strip_prefix(pyproject_dir).unwrap());
+            if source.is_dir() {
+                writer.add_directory(target)?;
+            } else {
+                writer.add_file(target, &source)?;
+            }
+        }
+    }
+
+    // Add readme, license
     if let Some(project) = pyproject.project.as_ref() {
         if let Some(pyproject_toml::ReadMe::RelativePath(readme)) = project.readme.as_ref() {
             writer.add_file(root_dir.join(readme), pyproject_dir.join(readme))?;
@@ -494,32 +668,40 @@ pub fn source_distribution(
         {
             writer.add_file(root_dir.join(license), pyproject_dir.join(license))?;
         }
-        if let Some(python_source) = pyproject.python_source() {
-            for entry in ignore::Walk::new(pyproject_dir.join(python_source)) {
-                let source = entry?.into_path();
-                let target = root_dir.join(source.strip_prefix(&pyproject_dir)?);
-                if source.is_dir() {
-                    writer.add_directory(target)?;
-                } else {
-                    writer.add_file(target, &source)?;
-                }
+    }
+
+    let mut include = |pattern| -> Result<()> {
+        println!("📦 Including files matching \"{}\"", pattern);
+        for source in glob::glob(&pyproject_dir.join(pattern).to_string_lossy())
+            .expect("No files found for pattern")
+            .filter_map(Result::ok)
+        {
+            let target = root_dir.join(source.strip_prefix(pyproject_dir).unwrap());
+            if source.is_dir() {
+                writer.add_directory(target)?;
+            } else {
+                writer.add_file(target, source)?;
             }
         }
-    }
+        Ok(())
+    };
+
+    #[allow(deprecated)]
     if let Some(include_targets) = pyproject.sdist_include() {
+        eprintln!(
+            "⚠️  Warning: `[tool.maturin.sdist-include]` is deprecated, please use `[tool.maturin.include]`"
+        );
         for pattern in include_targets {
-            println!("📦 Including files matching \"{}\"", pattern);
-            for source in glob::glob(&pyproject_dir.join(pattern).to_string_lossy())
-                .expect("No files found for pattern")
-                .filter_map(Result::ok)
-            {
-                let target = root_dir.join(&source.strip_prefix(pyproject_dir)?);
-                if source.is_dir() {
-                    writer.add_directory(target)?;
-                } else {
-                    writer.add_file(target, source)?;
-                }
-            }
+            include(pattern.as_str())?;
+        }
+    }
+
+    if let Some(glob_patterns) = pyproject.include() {
+        for pattern in glob_patterns
+            .iter()
+            .filter_map(|glob_pattern| glob_pattern.targets(Format::Sdist))
+        {
+            include(pattern)?;
         }
     }
 
