@@ -13,6 +13,7 @@ use std::env;
 use std::io;
 use std::path::{Path, PathBuf};
 use thiserror::Error;
+use tracing::debug;
 
 /// An account with a registry, possibly incomplete
 #[derive(Debug, clap::Parser)]
@@ -20,12 +21,12 @@ pub struct PublishOpt {
     /// The repository (package index) to upload the package to. Should be a section in the config file.
     ///
     /// Can also be set via MATURIN_REPOSITORY environment variable.
-    #[clap(short = 'r', long, env = "MATURIN_REPOSITORY", default_value = "pypi")]
+    #[arg(short = 'r', long, env = "MATURIN_REPOSITORY", default_value = "pypi")]
     repository: String,
     /// The URL of the registry where the wheels are uploaded to. This overrides --repository.
     ///
     /// Can also be set via MATURIN_REPOSITORY_URL environment variable.
-    #[clap(
+    #[arg(
         long = "repository-url",
         env = "MATURIN_REPOSITORY_URL",
         overrides_with = "repository"
@@ -36,16 +37,16 @@ pub struct PublishOpt {
     /// Can also be set via MATURIN_USERNAME environment variable.
     ///
     /// Set MATURIN_PYPI_TOKEN variable to use token-based authentication instead
-    #[clap(short, long, env = "MATURIN_USERNAME")]
+    #[arg(short, long, env = "MATURIN_USERNAME")]
     username: Option<String>,
     /// Password for pypi or your custom registry.
     ///
     /// Can also be set via MATURIN_PASSWORD environment variable.
-    #[clap(short, long)]
+    #[arg(short, long)]
     password: Option<String>,
     /// Continue uploading files if one already exists.
     /// (Only valid when uploading to PyPI. Other implementations may not support this.)
-    #[clap(long = "skip-existing")]
+    #[arg(long = "skip-existing")]
     skip_existing: bool,
 }
 
@@ -67,7 +68,7 @@ pub enum UploadError {
     UreqError(#[source] ureq::Error),
     /// The registry returned a "403 Forbidden"
     #[error("Username or password are incorrect")]
-    AuthenticationError,
+    AuthenticationError(String),
     /// Reading the wheel failed
     #[error("IO Error")]
     IoError(#[source] io::Error),
@@ -259,8 +260,9 @@ fn canonicalize_name(name: &str) -> String {
 }
 
 /// Uploads a single wheel to the registry
+#[allow(clippy::result_large_err)]
 pub fn upload(registry: &Registry, wheel_path: &Path) -> Result<(), UploadError> {
-    let hash_hex = hash_file(&wheel_path)?;
+    let hash_hex = hash_file(wheel_path)?;
 
     let dist = python_pkginfo::Distribution::new(wheel_path)
         .map_err(|err| UploadError::PkgInfoError(wheel_path.to_owned(), err))?;
@@ -321,7 +323,7 @@ pub fn upload(registry: &Registry, wheel_path: &Path) -> Result<(), UploadError>
     add_vec("requires_external", &metadata.requires_external);
     add_vec("project_urls", &metadata.project_urls);
 
-    let wheel = File::open(&wheel_path)?;
+    let wheel = File::open(wheel_path)?;
     let wheel_name = wheel_path
         .file_name()
         .expect("Wheel path has a file name")
@@ -335,7 +337,7 @@ pub fn upload(registry: &Registry, wheel_path: &Path) -> Result<(), UploadError>
     form.add_stream("content", &wheel, Some(wheel_name), None);
     let multipart_data = form.prepare().map_err(|e| e.error)?;
 
-    let encoded = base64::encode(&format!("{}:{}", registry.username, registry.password));
+    let encoded = base64::encode(format!("{}:{}", registry.username, registry.password));
 
     let http_proxy = env::var("HTTPS_PROXY")
         .or_else(|_| env::var("https_proxy"))
@@ -390,6 +392,7 @@ pub fn upload(registry: &Registry, wheel_path: &Path) -> Result<(), UploadError>
                     e
                 )
             });
+            debug!("Upload error response: {}", err_text);
             // Detect FileExistsError the way twine does
             // https://github.com/pypa/twine/blob/87846e5777b380d4704704a69e1f9a7a1231451c/twine/commands/upload.py#L30
             if status == 403 {
@@ -397,7 +400,7 @@ pub fn upload(registry: &Registry, wheel_path: &Path) -> Result<(), UploadError>
                     // Artifactory (https://jfrog.com/artifactory/)
                     Err(UploadError::FileExistsError(err_text))
                 } else {
-                    Err(UploadError::AuthenticationError)
+                    Err(UploadError::AuthenticationError(err_text))
                 }
             } else {
                 let status_string = status.to_string();
@@ -430,8 +433,18 @@ pub fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
 
         match upload_result {
             Ok(()) => (),
-            Err(UploadError::AuthenticationError) => {
-                println!("⛔ Username and/or password are wrong");
+            Err(UploadError::AuthenticationError(msg)) => {
+                let title_re = regex::Regex::new(r"<title>(.+?)</title>").unwrap();
+                let title = title_re
+                    .captures(&msg)
+                    .and_then(|c| c.get(1))
+                    .map(|m| m.as_str());
+                match title {
+                    Some(title) => {
+                        println!("⛔ {}", title);
+                    }
+                    None => println!("⛔ Username and/or password are wrong"),
+                }
 
                 #[cfg(feature = "keyring")]
                 {
@@ -451,7 +464,7 @@ pub fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
                     }
                 }
 
-                bail!("Username and/or password are wrong");
+                bail!("Username and/or password are possibly wrong");
             }
             Err(err) => {
                 let filename = i.file_name().unwrap_or(i.as_os_str());
@@ -464,7 +477,7 @@ pub fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
                         continue;
                     }
                 }
-                let filesize = fs::metadata(&i)
+                let filesize = fs::metadata(i)
                     .map(|x| ByteSize(x.len()).to_string())
                     .unwrap_or_else(|e| format!("Failed to get the filesize of {:?}: {}", &i, e));
                 return Err(err)
